/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.ServiceNotFoundException;

import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionDoesNotExistException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionRemoteErrorException;

/**
 * The Petals JMX Client API.
 * 
 * @author Christophe Deneux - Capgemini Sud
 */
public abstract class AbstractServiceClient {

    protected static final String JMX_FIELD_TYPE = "type";

    protected static final String JMX_FIELD_NAME = "name";

    protected static final String JMX_SERVICE_TYPE = "service";

    protected final String petalsDomain;

    protected ObjectName mbeanName = null;

    protected final MBeanServerConnection mBeanServerConnection;

    /**
     * Creates a new instance of {@link AbstractServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     */
    public AbstractServiceClient(final String petalsDomain, final MBeanServerConnection mBeanServerConnection) {
        this.petalsDomain = petalsDomain;
        this.mBeanServerConnection = mBeanServerConnection;
    }

    /**
     * Perform an action on the MBean service
     * 
     * @param action
     * @return
     * @throws PerformActionDoesNotExistException
     *             The operation invoked does not exist on the server side
     * @throws PerformActionRemoteErrorException
     *             An exception occurs during the remote processing
     * @throws PerformActionErrorException
     *             An exception occurs during the local processing
     */
    protected Object performAction(final String action, final Object[] objectsLoad, final String[] stringsLoad)
            throws PerformActionDoesNotExistException, PerformActionRemoteErrorException, PerformActionErrorException {

        return this.performAction(this.mbeanName, action, objectsLoad, stringsLoad);

    }

    protected Object performAction(final ObjectName mbeanName, final String action, final Object[] objectsLoad,
            final String[] stringsLoad)
            throws PerformActionDoesNotExistException, PerformActionRemoteErrorException, PerformActionErrorException {

        if ((objectsLoad == null && stringsLoad != null) || (objectsLoad != null && stringsLoad == null)
                || (objectsLoad != null && stringsLoad != null && objectsLoad.length != stringsLoad.length)) {
            throw new PerformActionErrorException(
                    "Inconsistency between parameters and the signature of the operation to invoke");
        }
        try {
            return this.mBeanServerConnection.invoke(mbeanName, action, objectsLoad, stringsLoad);
        } catch (final MBeanException e) {
            if (e.getCause() instanceof ServiceNotFoundException) {
                // The requested action does not exist on the server MBean side
                throw new PerformActionDoesNotExistException(action, mbeanName, e);
            } else {
                // bypass the MBean exception as it doesn't contains any
                // relevant information
                throw new PerformActionRemoteErrorException(e.getCause());
            }
        } catch (final Exception e) {
            throw new PerformActionErrorException(e);
        }
    }

    public Object getAttribute(final String attributeName) throws AttributeErrorException {

        return this.getAttribute(this.mbeanName, attributeName);
    }

    protected Object getAttribute(final ObjectName mbeanName, final String attributeName)
            throws AttributeErrorException {
        try {
            return this.mBeanServerConnection.getAttribute(mbeanName, attributeName);

        } catch (final AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (final InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (final MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (final ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (final IOException e) {
            throw new AttributeErrorException(e);
        }
    }

    public void setAttribute(final String name, final Object value) throws AttributeErrorException {
        try {
            this.mBeanServerConnection.setAttribute(this.mbeanName, new Attribute(name, value));
        } catch (final AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (final InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (final MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (final ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (final IOException e) {
            throw new AttributeErrorException(e);
        } catch (final InvalidAttributeValueException e) {
            throw new AttributeErrorException(e);
        }
    }

}
