/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.management.JMX;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentUnsupportedOperationException;
import org.ow2.petals.jmx.api.impl.configuration.component.RuntimeConfigurationComponentClientImpl;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class ComponentClientImpl extends AbstractServiceClient implements ComponentClient {

    private static final String RUNTIME_CONFIGURATION_JMX_NAME = "runtime_configuration_";

    private static final String RUNTIME_CONFIGURATION_JMX_TYPE = "custom";

    /**
     * Action : shutdown a component
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start a component
     */
    protected static final String START = "start";

    /**
     * Action : stop a component
     */
    protected static final String STOP = "stop";

    /**
     * Action : get current state
     */
    protected static final String GET_CURRENT_STATE = "getCurrentState";

    /**
     * Action : reload placeholders configuration
     */
    protected static final String RELOAD_PLACEHOLDERS = "reloadPlaceHolders";

    /**
     * Name of the runtime configuration MBean associated to the current component
     */
    private final ObjectName runtimeConfMbeanName;

    /**
     * Creates a new instance of {@link ComponentClient}
     * 
     * @param petalsDomain
     * @param componentName
     * @param mBeanServerConnection
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     */
    public ComponentClientImpl(final String petalsDomain, final String componentName,
            final MBeanServerConnection mBeanServerConnection) throws ComponentDoesNotExistException,
            ComponentErrorException {
        this(retrieveComponentMBean(componentName, mBeanServerConnection), mBeanServerConnection);
    }

    private static ObjectName retrieveComponentMBean(final String componentName,
            final MBeanServerConnection mBeanServerConnection)
            throws ComponentDoesNotExistException, ComponentErrorException {
        try {
            final String sePattern = "*:type=engine,name=" + componentName;
            final ObjectName seObjName = new ObjectName(sePattern);
            final Set<ObjectName> seObjNames = mBeanServerConnection.queryNames(seObjName, null);
            if (seObjNames != null && seObjNames.size() == 1) {
                return seObjNames.iterator().next();
            } else {
                final String bcPattern = "*:type=binding,name=" + componentName;
                final ObjectName bcObjName = new ObjectName(bcPattern);
                final Set<ObjectName> bcObjNames = mBeanServerConnection.queryNames(bcObjName, null);
                if (bcObjNames != null && bcObjNames.size() == 1) {
                    return bcObjNames.iterator().next();
                } else {
                    throw new ComponentDoesNotExistException(componentName);
                }
            }
        } catch (final MalformedObjectNameException e) {
            throw new ComponentErrorException(e);
        } catch (final IOException e) {
            throw new ComponentErrorException(e);
        }
    }

    private static ObjectName retrieveRuntimeConfigurationMBean(final String componentName, final String petalsDomain,
            final MBeanServerConnection mBeanServerConnection) throws ComponentErrorException {
        try {
            /*
             * The RUNTIME_CONFIGURATION_JMX_NAME represent the static name part of the focused MBean like
             * "runtime_configuration_", and componentName the dynamic name part of this MBean like
             * "petals-bc-filetransfer". Finally all the MBean can be: runtime_configuration_petals-bc-filetransfer
             */
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, RUNTIME_CONFIGURATION_JMX_NAME + componentName);
            attributes.put(JMX_FIELD_TYPE, RUNTIME_CONFIGURATION_JMX_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                return objNames.iterator().next();
            } else {
                return null;
            }
        } catch (final MalformedObjectNameException e) {
            throw new ComponentErrorException(e);
        } catch (final IOException e) {
            throw new ComponentErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link ComponentClient}
     * 
     * @param componentObjectName
     * @param mBeanServerConnection
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     */
    public ComponentClientImpl(final ObjectName componentObjectName, final MBeanServerConnection mBeanServerConnection)
            throws ComponentDoesNotExistException,
            ComponentErrorException {
        super(componentObjectName.getDomain(), mBeanServerConnection);

        this.mbeanName = componentObjectName;

        final String componentName = this.mbeanName.getKeyProperty(JMX_FIELD_NAME);

        this.runtimeConfMbeanName = retrieveRuntimeConfigurationMBean(componentName, petalsDomain,
                mBeanServerConnection);
    }

    @Override
    public void start() throws ComponentErrorException {
        try {
            this.performAction(START, null, null);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }
    }

    @Override
    public void stop() throws ComponentErrorException {
        try {
            this.performAction(STOP, null, null);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }
    }

    @Override
    public void shutdown() throws ComponentErrorException {
        try {
            this.performAction(SHUTDOWN, null, null);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }

    }

    @Override
    public State getCurrentState() throws ComponentErrorException {

        try {
            final String state = (String) this.performAction(GET_CURRENT_STATE, null, null);
            return State.parse(state);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentErrorException("Unexpected result type", e);
        }
    }

    @Override
    public void reloadConfiguration() throws ComponentErrorException {
        if (this.runtimeConfMbeanName != null) {
            try {
                this.performAction(this.runtimeConfMbeanName, RELOAD_PLACEHOLDERS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new ComponentErrorException(e);
            }
        } else {
            throw new ComponentUnsupportedOperationException(RELOAD_PLACEHOLDERS);
        }
    }

    @Override
    public RuntimeConfigurationComponentClient getRuntimeConfigurationClient()
            throws ConfigurationComponentDoesNotExistException {
        if (this.runtimeConfMbeanName != null) {
            return new RuntimeConfigurationComponentClientImpl(this.runtimeConfMbeanName, this.petalsDomain,
                    this.mBeanServerConnection);
        } else {
            throw new ConfigurationComponentDoesNotExistException();
        }
    }

    @Override
    public <T> T getRuntimeConfigurationClient(final Class<T> clazz)
            throws ConfigurationComponentDoesNotExistException {
        if (this.runtimeConfMbeanName != null) {
            return JMX.newMBeanProxy(this.mBeanServerConnection, this.runtimeConfMbeanName, clazz);
        } else {
            throw new ConfigurationComponentDoesNotExistException();
        }
    }
}
