/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.management.JMX;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.InstallationConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentErrorException;
import org.ow2.petals.jmx.api.impl.configuration.component.InstallationConfigurationComponentClientImpl;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component installer.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class InstallerComponentClientImpl extends AbstractServiceClient implements
        InstallerComponentClient {

    /**
     * Action : init
     */
    protected static final String INIT = "init";

    /**
     * Action : install
     */
    protected static final String INSTALL = "install";

    /**
     * Action : uninstall
     */
    protected static final String UNINSTALL = "uninstall";

    /**
     * Action : isInstalled
     */
    protected static final String IS_INSTALLED = "isInstalled";

    /**
     * Action : getInstallRoot
     */
    protected static final String GET_INSTALL_ROOT = "getInstallRoot";

    /**
     * Action : getInstallerConfigurationMBean
     */
    protected static final String GET_INSTALLER_CONFIGURATION_MBEAN = "getInstallerConfigurationMBean";

    /**
     * JMX type
     */
    private static final String INSTALLER_COMPONENT_JMX_TYPE = "installer";

    private InstallationConfigurationComponentClient configurationInstaller = null;

    /**
     * Creates a new instance of {@link InstallerComponentClient} finding
     * the MBean associated to the component name
     * 
     */
    public InstallerComponentClientImpl(final String petalsDomain, final String componentName,
            final MBeanServerConnection mBeanServerConnection)
            throws InstallerComponentDoesNotExistException, ComponentErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, componentName);
            attributes.put(JMX_FIELD_TYPE, INSTALLER_COMPONENT_JMX_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new InstallerComponentDoesNotExistException(componentName);
            }
        } catch (final MalformedObjectNameException e) {
            throw new ComponentErrorException(e);
        } catch (final IOException e) {
            throw new ComponentErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link InstallerComponentClient} from its
     * MBean
     * 
     */
    public InstallerComponentClientImpl(final String petalsDomain, final ObjectName objectName,
            final MBeanServerConnection mBeanServerConnection) {

        super(petalsDomain, mBeanServerConnection);
        this.mbeanName = objectName;

    }

    @Override
    public void install() throws InstallerComponentErrorException {
        try {
            this.performAction(INSTALL, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    @Override
    public void uninstall() throws InstallerComponentErrorException {
        try {
            this.performAction(UNINSTALL, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    @Override
    public boolean isInstalled() throws InstallerComponentErrorException {

        try {
            try {
                final Boolean result = (Boolean) this.performAction(IS_INSTALLED, null, null);
                return result.booleanValue();
            } catch (final ClassCastException e) {
                throw new PerformActionErrorException("Unexpected result type", e);
            }
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    @Override
    public String getInstallRoot() throws InstallerComponentErrorException {

        try {
            try {
                return (String) this.performAction(GET_INSTALL_ROOT, null, null);
            } catch (final ClassCastException e) {
                throw new PerformActionErrorException("Unexpected result type", e);
            }
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    @Override
    public ObjectName getMBeanName() {
        return this.mbeanName;
    }
    
    @Override
    public InstallationConfigurationComponentClient getInstallationConfigurationClient()
            throws ConfigurationComponentDoesNotExistException, ConfigurationComponentErrorException,
            InstallerComponentErrorException {
        if (this.configurationInstaller == null) {
            try {
                final ObjectName result = (ObjectName) this
                        .performAction(GET_INSTALLER_CONFIGURATION_MBEAN, null,
                        null);
                if (result == null) {
                    this.configurationInstaller = null;
                }
                else {
                    this.configurationInstaller = new InstallationConfigurationComponentClientImpl(result,
                            this.petalsDomain, this.mBeanServerConnection);
                }

            } catch (final PerformActionErrorException e) {
                throw new InstallerComponentErrorException(e);
            }
        }

        return this.configurationInstaller;
    }

    @Override
    public <T> T getInstallationConfigurationClient(Class<T> clazz)
            throws ConfigurationComponentDoesNotExistException, InstallerComponentErrorException {
        try {
            final ObjectName result = (ObjectName) this.performAction(GET_INSTALLER_CONFIGURATION_MBEAN, null, null);
            if (result == null) {
                throw new ConfigurationComponentDoesNotExistException();
            } else {
                return JMX.newMBeanProxy(this.mBeanServerConnection, result, clazz);
            }
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }
}
