/**
 * Copyright (c) 2010-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.clientserverapi.system.logging.exception.LoggingServiceException;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of the JMX Logger service.
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class LoggerServiceClientImpl extends AbstractServiceClient implements LoggerServiceClient {

    /**
     * Name of the associated MBean provided by the JVM (Java Logging)
     */
    private static final String JAVA_LOGGER_SERVICE_MBEAN_NAME = "Logging";

    /**
     * Action : set level for Logger
     */
    private static final String SET_LOGGER_LEVEL = "setLoggerLevel";

    /**
     * Action : get level for Loggers
     */
    private static final String GET_LOGGER_LEVEL = "getLoggerLevel";

    /**
     * Action : get all Levels
     */
    private static final String GET_LEVELS = "getLevels";

    /**
     * Action : get all Loggers
     */
    private static final String GET_LOGGERS = "getLoggers";

    private final ObjectName javaLoggingMbeanName;

    public LoggerServiceClientImpl(final String petalsDomain,
            final MBeanServerConnection mBeanServerConnection) throws LoggerServiceErrorException,
            LoggerServiceDoesNotExistException {
        super(petalsDomain, mBeanServerConnection);

        try {
            // MBean: PetalsLogger
            {
                final Hashtable<String, String> attributes = new Hashtable<String, String>();
                attributes.put(JMX_FIELD_NAME, LoggerServiceClient.LOGGER_SERVICE_MBEAN_NAME);
                attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
                final ObjectName objName = new ObjectName(petalsDomain, attributes);
                final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName,
                        null);
                if (objNames != null && objNames.size() == 1) {
                    this.mbeanName = objNames.iterator().next();
                } else {
                    throw new LoggerServiceDoesNotExistException();
                }
            }

            // MBean: Java Util Logging
            {
                final Hashtable<String, String> attributes = new Hashtable<String, String>();
                attributes.put(JMX_FIELD_TYPE,
                        LoggerServiceClientImpl.JAVA_LOGGER_SERVICE_MBEAN_NAME);
                final ObjectName objName = new ObjectName("java.util.logging", attributes);
                final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName,
                        null);
                if (objNames != null && objNames.size() == 1) {
                    this.javaLoggingMbeanName = objNames.iterator().next();
                } else {
                    throw new LoggerServiceDoesNotExistException();
                }
            }
        } catch (final MalformedObjectNameException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final IOException e) {
            throw new LoggerServiceErrorException(e);
        }
    }

    @Override
    public void setLoggerLevel(final String logger, final String level)
            throws LoggerServiceErrorException {

        final Object[] objectsLoad = new Object[] { logger, level };
        final String[] stringsLoad = new String[] { String.class.getName(), String.class.getName() };
        try {
            this.performAction(this.javaLoggingMbeanName, SET_LOGGER_LEVEL, objectsLoad,
                    stringsLoad);
        } catch (final PerformActionErrorException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LoggerServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public String getLoggerLevel(final String logger) throws LoggerServiceErrorException {

        final Object[] objectsLoad = new Object[] { logger };
        final String[] stringsLoad = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(this.javaLoggingMbeanName, GET_LOGGER_LEVEL,
                    objectsLoad, stringsLoad);
        } catch (final PerformActionErrorException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LoggerServiceErrorException("Unexpected result type", e);
        }

    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * If an error occurs performing the JMX operation, the cause of the {@link LoggingServiceException} will be
     * {@link LoggerServiceErrorException}.
     * </p>
     */
    @Override
    public String[] getLevels() throws LoggingServiceException {

        try {
            try {
                return (String[]) this.performAction(GET_LEVELS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new LoggerServiceErrorException(e);
            } catch (final ClassCastException e) {
                throw new LoggerServiceErrorException("Unexpected result type", e);
            }
        } catch (final LoggerServiceErrorException e) {
            throw new LoggingServiceException(e);
        }
    }
    
    /**
     * {@inheritDoc}
     * 
     * <p>
     * If an error occurs performing the JMX operation, the cause of the {@link LoggingServiceException} will be
     * {@link LoggerServiceErrorException}.
     * </p>
     */
    @Override
    public String[][] getLoggers() throws LoggingServiceException {

        try {
            try {
                return (String[][]) this.performAction(GET_LOGGERS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new LoggerServiceErrorException(e);
            } catch (final ClassCastException e) {
                throw new LoggerServiceErrorException("Unexpected result type", e);
            }
        } catch (final LoggerServiceErrorException e) {
            throw new LoggingServiceException(e);
        }
    }
}
