/**
 * Copyright (c) 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.component.soap;

import java.io.IOException;
import java.net.InetAddress;
import java.util.Set;

import javax.management.InstanceNotFoundException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.NoDefectListenerException;
import org.ow2.petals.jmx.api.impl.monitoring.defect.AbstractDefectListener;
import org.ow2.petals.jmx.api.impl.monitoring.defect.DefectNotif;

/**
 * Petals BC SOAP defect listener.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class PetalsBcSoapMonitoringDefectListenerImpl extends AbstractDefectListener implements
        NotificationListener {

    /**
     * JMX base name
     */
    private static final String SERVICE_JMX_BASENAME = "monitoring_";

    /**
     * The defect listener
     */
    private DefectListener defectListener = null;

    /**
     * The defect listener
     */
    private final ObjectName mbeanName;

    /**
     * The identifier of the defect emitter
     */
    private final String source;

    /**
     * Creates a new instance of
     * {@link PetalsBcSoapMonitoringDefectListenerImpl}
     * 
     * @param petalsDomain
     * @param componentIdentifier
     *            The unique identifier of the Petals BC SOAP to subscribe
     * @param hostanme
     *            the hostname holding the component
     * @param mBeanServerConnection
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ComponentMonitoringServiceErrorException
     */
    public PetalsBcSoapMonitoringDefectListenerImpl(final String petalsDomain,
            final String componentIdentifier, final InetAddress hostname,
            final MBeanServerConnection mBeanServerConnection)
            throws ComponentMonitoringServiceDoesNotExistException,
            ComponentMonitoringServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final String pattern = "*:type=custom,name=" + SERVICE_JMX_BASENAME
                    + componentIdentifier;
            this.source = componentIdentifier + '@' + hostname.getHostName();
            final ObjectName objName = new ObjectName(pattern);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new ComponentMonitoringServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final IOException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        }
    }

    @Override
    public void subscribe(final DefectListener defectListener, final DefectFilter defectFilter)
            throws DefectListenerException, DefectEmitterDoesNotExistsException {

        this.defectListener = defectListener;
        // TODO: Set a NotificationFilter associated to the defect
        // filter
        try {
            this.mBeanServerConnection.addNotificationListener(this.mbeanName, this, null, null);
        } catch (final IOException e) {
            throw new DefectListenerException(e);
        } catch (final InstanceNotFoundException e) {
            throw new DefectEmitterDoesNotExistsException(e, this.mbeanName.toString());
        }
    }

    @Override
    public void unsubscribe() throws NoDefectListenerException,
            DefectEmitterDoesNotExistsException, DefectListenerException {

        try {
            this.mBeanServerConnection.removeNotificationListener(this.mbeanName, this);
        } catch (final InstanceNotFoundException e) {
            throw new DefectEmitterDoesNotExistsException(e, this.mbeanName.toString());
        } catch (final ListenerNotFoundException e) {
            throw new NoDefectListenerException(e);
        } catch (final IOException e) {
            throw new DefectListenerException(e);
        }
    }

    @Override
    public void handleNotification(final Notification notification, final Object handback) {

        if (this.defectListener != null) {
            this.defectListener.onDefect(new DefectNotif(notification, this.source));
        }
    }
}
