/**
 * Copyright (c) 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.component.soap;

import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServerConnection;

import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.OutgoingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsClientPoolKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsRequestExecutionStatus;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.impl.monitoring.component.framework.ComponentMonitoringServiceClientImpl;
import org.ow2.petals.jmx.api.impl.monitoring.object.ThreadPoolMetricsImpl;

/**
 * Client API of the JMX service providing Petals BC Soap monitoring metrics.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class PetalsBcSoapMonitoringServiceClientImpl extends ComponentMonitoringServiceClientImpl
        implements PetalsBcSoapMonitoringServiceClient {

    /**
     * Attribute : HttpServerThreadPoolMinSize
     */
    private static final String HTTP_SERVER_THREAD_POOL_MIN_SIZE = "HttpServerThreadPoolMinSize";

    /**
     * Attribute : HttpServerThreadPoolMaxSize
     */
    private static final String HTTP_SERVER_THREAD_POOL_MAX_SIZE = "HttpServerThreadPoolMaxSize";

    /**
     * Attribute : HttpServerThreadPoolActiveThreadsCurrent
     */
    private static final String HTTP_SERVER_THREAD_POOL_ACTIVE_THREADS_CURRENT = "HttpServerThreadPoolActiveThreadsCurrent";

    /**
     * Attribute : HttpServerThreadPoolActiveThreadsMax
     */
    private static final String HTTP_SERVER_THREAD_POOL_ACTIVE_THREADS_MAX = "HttpServerThreadPoolActiveThreadsMax";

    /**
     * Attribute : HttpServerThreadPoolIdleThreadsCurrent
     */
    private static final String HTTP_SERVER_THREAD_POOL_IDLE_THREADS_CURRENT = "HttpServerThreadPoolIdleThreadsCurrent";

    /**
     * Attribute : HttpServerThreadPoolIdleThreadsMax
     */
    private static final String HTTP_SERVER_THREAD_POOL_IDLE_THREADS_MAX = "HttpServerThreadPoolIdleThreadsMax";

    /**
     * Attribute : HttpServerThreadPoolQueuedRequestsCurrent
     */
    private static final String HTTP_SERVER_THREAD_POOL_QUEUED_REQUESTS_CURRENT = "HttpServerThreadPoolQueuedRequestsCurrent";

    /**
     * Attribute : HttpServerThreadPoolQueuedRequestsMax
     */
    private static final String HTTP_SERVER_THREAD_POOL_QUEUED_REQUESTS_MAX = "HttpServerThreadPoolQueuedRequestsMax";

    /**
     * Attribute : IncomingWsRequestsCounter
     */
    private static final String INCOMING_WS_REQUESTS_COUNT = "IncomingWsRequestsCounter";

    /**
     * Attribute : OutgoingWsRequestsCounter
     */
    private static final String OUTGOING_WS_REQUESTS_COUNT = "OutgoingWsRequestsCounter";

    /**
     * Attribute : InformationURLsCounter
     */
    private static final String INFORMATION_REQUESTS_COUNT = "InformationURLsCounter";

    /**
     * Attribute : UnknownURLsCounter
     */
    private static final String UNKNOWN_REQUESTS_COUNT = "UnknownURLsCounter";

    /**
     * Attribute : ServiceContractsCounter
     */
    private static final String SERVICE_CONTRACTS_REQUESTS_COUNT = "ServiceContractsCounter";

    /**
     * Attribute : IncomingWsRequestsResponseTimeRel
     */
    private static final String INCOMING_WS_REQUESTS_RESPONSE_TIME_REL = "IncomingWsRequestsResponseTimeRel";

    /**
     * Attribute : OutgoingWsRequestsResponseTimeRel
     */
    private static final String OUTGOING_WS_REQUESTS_RESPONSE_TIME_REL = "OutgoingWsRequestsResponseTimeRel";

    /**
     * Attribute : IncomingWsRequestsResponseTimeRelGroupedBySvcAndExecStatus
     */
    private static final String INCOMING_WS_REQUESTS_RESPONSE_TIME_REL_BY_SVC_AND_EXEC_STATUS = "IncomingWsRequestsResponseTimeRelGroupedBySvcAndExecStatus";

    /**
     * Attribute : OutgoingWsRequestsResponseTimeRelGroupedBySvcAndExecStatus
     */
    private static final String OUTGOING_WS_REQUESTS_RESPONSE_TIME_REL_BY_SVC_AND_EXEC_STATUS = "OutgoingWsRequestsResponseTimeRelGroupedBySvcAndExecStatus";

    /**
     * Attribute : WsClientPoolClientsInUseCurrent
     */
    private static final String WS_CLIENT_POOL_CLIENTS_IN_USE_CURRENT = "WsClientPoolClientsInUseCurrent";

    /**
     * Attribute : WsClientPoolClientsInUseMax
     */
    private static final String WS_CLIENT_POOL_CLIENTS_IN_USE_MAX = "WsClientPoolClientsInUseMax";

    /**
     * Attribute : WsClientPoolExhaustions
     */
    private static final String WS_CLIENT_POOL_EXHAUSTIONS = "WsClientPoolExhaustions";

    /**
     * Creates a new instance of {@link PetalsBcSoapMonitoringServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ComponentMonitoringServiceErrorException
     */
    public PetalsBcSoapMonitoringServiceClientImpl(final String petalsDomain, final String componentIdentifier,
            final MBeanServerConnection mBeanServerConnection)
                    throws ComponentMonitoringServiceDoesNotExistException, ComponentMonitoringServiceErrorException {

        super(petalsDomain, componentIdentifier, mBeanServerConnection);
    }

    @Override
    public Map<IncomingWsRequestKey, Long> getIncomingWsRequestsCount()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long> values = (Map<String[], Long>) this.getAttribute(INCOMING_WS_REQUESTS_COUNT);

            final Map<IncomingWsRequestKey, Long> results = new HashMap<IncomingWsRequestKey, Long>(values.size());
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(new IncomingWsRequestKey(key[IncomingWsRequestIndexes.WS_PATH_IDX],
                        key[IncomingWsRequestIndexes.OPERATION_IDX], key[IncomingWsRequestIndexes.CLIENT_IP_ADRESS_IDX],
                        WsRequestExecutionStatus.parse(key[IncomingWsRequestIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;

        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Long getInformationHttpRequestsCount() throws ComponentMonitoringServiceErrorException {

        try {
            return (Long) this.getAttribute(INFORMATION_REQUESTS_COUNT);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Long getUnknownHttpRequestsCount() throws ComponentMonitoringServiceErrorException {

        try {
            return (Long) this.getAttribute(UNKNOWN_REQUESTS_COUNT);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<String, Long> getServiceContractsHttpRequestsCount() throws ComponentMonitoringServiceErrorException {

        try {
            return (Map<String, Long>) this.getAttribute(SERVICE_CONTRACTS_REQUESTS_COUNT);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<IncomingWsRequestKey, Long[]> getIncomingWsRequestsResponseTimes()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(INCOMING_WS_REQUESTS_RESPONSE_TIME_REL);

            final Map<IncomingWsRequestKey, Long[]> results = new HashMap<IncomingWsRequestKey, Long[]>(values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(new IncomingWsRequestKey(key[IncomingWsRequestIndexes.WS_PATH_IDX],
                        key[IncomingWsRequestIndexes.OPERATION_IDX], key[IncomingWsRequestIndexes.CLIENT_IP_ADRESS_IDX],
                        WsRequestExecutionStatus.parse(key[IncomingWsRequestIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<IncomingWsRequestKey, Long[]> getIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(INCOMING_WS_REQUESTS_RESPONSE_TIME_REL_BY_SVC_AND_EXEC_STATUS);

            final Map<IncomingWsRequestKey, Long[]> results = new HashMap<IncomingWsRequestKey, Long[]>(values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(new IncomingWsRequestKey(key[0], key[1], null, WsRequestExecutionStatus.parse(key[2])),
                        entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public ThreadPoolMetrics getHttpServerThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {

        final ThreadPoolMetricsImpl httpServerThreadPoolMetrics = new ThreadPoolMetricsImpl();

        // Min size
        try {
            httpServerThreadPoolMetrics.setMinSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_MIN_SIZE));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Max size
        try {
            httpServerThreadPoolMetrics.setMaxSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_MAX_SIZE));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Active thread current size
        try {
            httpServerThreadPoolMetrics
                    .setActiveThreadsSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_ACTIVE_THREADS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Active thread max size
        try {
            httpServerThreadPoolMetrics
                    .setActiveThreadsMaxSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_ACTIVE_THREADS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Idle thread size
        try {
            httpServerThreadPoolMetrics
                    .setIdleThreadsSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_IDLE_THREADS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Idle thread max size
        try {
            httpServerThreadPoolMetrics
                    .setIdleThreadsMaxSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_IDLE_THREADS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Enqueued Size
        try {
            httpServerThreadPoolMetrics
                    .setEnqueuedRequestsSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_QUEUED_REQUESTS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        // Enqueued Max Size
        try {
            httpServerThreadPoolMetrics
                    .setEnqueuedRequestsMaxSize((Long) this.getAttribute(HTTP_SERVER_THREAD_POOL_QUEUED_REQUESTS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }

        return httpServerThreadPoolMetrics;
    }

    @Override
    public Map<WsClientPoolKey, Long> getWsClientPoolClientsInUseMax() throws ComponentMonitoringServiceErrorException {

        try {
            return PetalsBcSoapMonitoringServiceClientImpl.convertWsClientsPoolValues(
                    (Map<String[], Long>) this.getAttribute(WS_CLIENT_POOL_CLIENTS_IN_USE_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<WsClientPoolKey, Long> getWsClientPoolClientsInUseCurrent()
            throws ComponentMonitoringServiceErrorException {

        try {
            return PetalsBcSoapMonitoringServiceClientImpl.convertWsClientsPoolValues(
                    (Map<String[], Long>) this.getAttribute(WS_CLIENT_POOL_CLIENTS_IN_USE_CURRENT));

        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<WsClientPoolKey, Long> getWsClientPoolExhaustions() throws ComponentMonitoringServiceErrorException {

        try {
            return PetalsBcSoapMonitoringServiceClientImpl
                    .convertWsClientsPoolValues((Map<String[], Long>) this.getAttribute(WS_CLIENT_POOL_EXHAUSTIONS));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    /**
     * Converts a value map returned by the JMX API about WS-clients and containing a key as String array into a value
     * map where the String array was converted into {@link WsClientPoolKey}.
     * 
     * @param values
     *            The value map returned by the JMX API
     * @return The converted value map
     */
    private static final Map<WsClientPoolKey, Long> convertWsClientsPoolValues(final Map<String[], Long> values) {
        final Map<WsClientPoolKey, Long> results = new HashMap<WsClientPoolKey, Long>(values.size());
        for (final Map.Entry<String[], Long> entry : values.entrySet()) {
            final String[] key = entry.getKey();
            results.put(new WsClientPoolKey(key[WsClientPoolIndexes.SERVICE_IDX],
                    key[WsClientPoolIndexes.OPERATION_IDX], key[WsClientPoolIndexes.MEP_IDX]), entry.getValue());
        }
        return results;
    }

    @Override
    public Map<OutgoingWsRequestKey, Long> getOutgoingWsRequestsCount()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long> values = (Map<String[], Long>) this.getAttribute(OUTGOING_WS_REQUESTS_COUNT);

            final Map<OutgoingWsRequestKey, Long> results = new HashMap<OutgoingWsRequestKey, Long>(values.size());
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(
                        new OutgoingWsRequestKey(key[OutgoingWsRequestIndexes.WS_URL],
                                key[OutgoingWsRequestIndexes.OPERATION_IDX], key[OutgoingWsRequestIndexes.MEP_IDX],
                                WsRequestExecutionStatus.parse(key[OutgoingWsRequestIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;

        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<OutgoingWsRequestKey, Long[]> getOutgoingWsRequestsResponseTimes()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(OUTGOING_WS_REQUESTS_RESPONSE_TIME_REL);

            final Map<OutgoingWsRequestKey, Long[]> results = new HashMap<OutgoingWsRequestKey, Long[]>(values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(
                        new OutgoingWsRequestKey(key[OutgoingWsRequestIndexes.WS_URL],
                                key[OutgoingWsRequestIndexes.OPERATION_IDX], key[OutgoingWsRequestIndexes.MEP_IDX],
                                WsRequestExecutionStatus.parse(key[IncomingWsRequestIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }

    @Override
    public Map<OutgoingWsRequestKey, Long[]> getOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(OUTGOING_WS_REQUESTS_RESPONSE_TIME_REL_BY_SVC_AND_EXEC_STATUS);

            final Map<OutgoingWsRequestKey, Long[]> results = new HashMap<OutgoingWsRequestKey, Long[]>(values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(new OutgoingWsRequestKey(key[0], key[1], null, WsRequestExecutionStatus.parse(key[2])),
                        entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type", e);
        }
    }
}
