/**
 * Copyright (c) 2011-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.net.MalformedURLException;
import java.net.URL;

import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.RuntimeOperationsException;
import javax.management.modelmbean.InvalidTargetObjectTypeException;

import org.junit.Test;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.impl.mbean.InstallationService;
import org.ow2.petals.jmx.api.impl.mbean.InstallationServiceResults;

/**
 * <p>
 * Unit tests of {@link InstallationServiceClientImpl}
 * </p>
 * <p>
 * Some unit tests check that JMX client interfaces are aligned with the JMX server MBean part. This unit tests
 * implement {@link InstallationServiceClient} to avoid to forget some methods. A unit test will only invoke the
 * associated method of {@link InstallationServiceClient}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * @author Mathieu Carrolle - EBM WebSourcing
 * 
 */
public class InstallationServiceClientImplTest extends AbstractServiceClientImpl implements InstallationServiceClient {

    /**
     * <p>
     * Check {@link InstallationServiceClient#loadInstaller(String)} against a component loader that was not loaded
     * previously, ie when the
     * {@link org.ow2.petals.clientserverapi.jbi.management.installation.InstallationService#loadInstaller(String)}
     * return <code>null</code>.
     * </p>
     * <p>
     * Expected result: an error occurs about the unexisting component
     * </p>
     */
    @Test(expected = InstallerComponentDoesNotExistException.class)
    public void testLoadInstaller_NotLoaded() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException,
            InstallerComponentDoesNotExistException, InstallationServiceDoesNotExistException, ConnectionErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()
                .setLoadInstallerResult(null)));

        final String inexistingComponentId = "inexisting-component-id";
        try {
            this.jmxClient.getInstallationServiceClient().loadInstaller(inexistingComponentId);
            fail("The exception '" + InstallerComponentDoesNotExistException.class.getName() + "' is not thrown.");
        } catch (final InstallerComponentDoesNotExistException e) {
            // NOP
            assertEquals(inexistingComponentId, e.getComponentName());
            throw e;
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#forceUnloadInstaller(String)}
     */
    @Test
    public void testJMXCompliance_ForceUnloadInstaller() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.forceUnloadInstaller("componentName");

    }

    @Override
    public boolean forceUnloadInstaller(final String componentName) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().forceUnloadInstaller(componentName);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of
     * {@link InstallationServiceClientImpl#getInstalledComponentsForSharedlibrary(String)}
     */
    @Test
    public void testJMXCompliance_GetInstalledComponentsForSharedlibrary() throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.getInstalledComponentsForSharedlibrary("sharedLibraryName", "sharedLibraryVersion");

    }

    @Override
    public String[] getInstalledComponentsForSharedlibrary(final String sharedLibraryName,
            final String sharedLibraryVersion)
            throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().getInstalledComponentsForSharedlibrary(
                    sharedLibraryName, sharedLibraryVersion);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#getInstalledSharedLibraries()}
     */
    @Test
    public void testJMXCompliance_GetInstalledSharedLibraries() throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.getInstalledSharedLibraries();

    }

    @Override
    public String[][] getInstalledSharedLibraries() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().getInstalledSharedLibraries();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#getInstallers()}
     */
    @Test
    public void testJMXCompliance_GetInstallers() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.getInstallers();

    }

    @Override
    public String[] getInstallers() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().getInstallers();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#installSharedLibrary(URL)}
     */
    @Test
    public void testJMXCompliance_InstallSharedLibrary() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException, MalformedURLException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.installSharedLibrary(new URL("http://localhost"));

    }

    @Override
    public String installSharedLibrary(final URL sharedLibraryURL) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().installSharedLibrary(sharedLibraryURL);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#loadInstaller(String)}
     */
    @Test
    public void testJMXCompliance_LoadInstaller() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException,
            InstallerComponentDoesNotExistException, MalformedObjectNameException, NullPointerException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()
                .setLoadInstallerResult(new ObjectName("Petals:name=dummy,type=service"))));
        this.loadInstaller("componentName");

    }

    @Override
    public InstallerComponentClient loadInstaller(final String componentName)
            throws InstallerComponentDoesNotExistException, InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().loadInstaller(componentName);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#loadNewInstaller(URL)}
     */
    @Test
    public void testJMXCompliance_LoadNewInstaller() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException, MalformedURLException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.loadNewInstaller(new URL("http://localhost"));

    }

    @Override
    public InstallerComponentClient loadNewInstaller(final URL packageURL) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().loadNewInstaller(packageURL);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#shutdownAllComponents()}
     */
    @Test
    public void testJMXCompliance_ShutdownAllComponents() throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.shutdownAllComponents();

    }

    @Override
    public String[] shutdownAllComponents() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().shutdownAllComponents();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#shutdownComponent(String)}
     */
    @Test
    public void testJMXCompliance_ShutdownComponent() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.shutdownComponent("componentName");

    }

    @Override
    public boolean shutdownComponent(final String componentName) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().shutdownComponent(componentName);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#startAllComponents()}
     */
    @Test
    public void testJMXCompliance_StartAllComponents() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.startAllComponents();

    }

    @Override
    public String[] startAllComponents() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().startAllComponents();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#startComponent(String)}
     */
    @Test
    public void testJMXCompliance_StartComponent() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.startComponent("componentName");

    }

    @Override
    public boolean startComponent(final String componentName) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().startComponent(componentName);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#stopAllComponents()}
     */
    @Test
    public void testJMXCompliance_StopAllComponents() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.stopAllComponents();

    }

    @Override
    public String[] stopAllComponents() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().stopAllComponents();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#stopComponent(String)}
     */
    @Test
    public void testJMXCompliance_StopComponent() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.stopComponent("componentName");

    }

    @Override
    public boolean stopComponent(final String componentName) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().startComponent(componentName);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#uninstallAllComponents()}
     */
    @Test
    public void testJMXCompliance_UninstallAllComponents() throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.uninstallAllComponents();

    }

    @Override
    public String[] uninstallAllComponents() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().uninstallAllComponents();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#uninstallAllSharedLibrary()}
     */
    @Test
    public void testJMXCompliance_UninstallAllSharedLibrary() throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.uninstallAllSharedLibrary();

    }

    @Override
    public String[] uninstallAllSharedLibrary() throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().uninstallAllSharedLibrary();
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#uninstallSharedLibrary(String)}
     */
    @Test
    public void testJMXCompliance_UninstallSharedLibrary() throws NotCompliantMBeanException,
            RuntimeOperationsException, InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.uninstallSharedLibrary("sharedLibraryName", "sharedLibraryVersion");

    }

    @Override
    public boolean uninstallSharedLibrary(final String sharedLibraryName, final String sharedLibraryVersion)
            throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().uninstallSharedLibrary(sharedLibraryName,
                    sharedLibraryVersion);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#unloadAllInstallers(boolean)}
     */
    @Test
    public void testJMXCompliance_UnloadAllInstallers() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.unloadAllInstallers(false);

    }

    @Override
    public String[] unloadAllInstallers(final boolean force) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().unloadAllInstallers(force);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * Check the interface consistency of {@link InstallationServiceClientImpl#unloadInstaller(String)}
     */
    @Test
    public void testJMXCompliance_UnloadInstaller() throws NotCompliantMBeanException, RuntimeOperationsException,
            InstanceNotFoundException, InstanceAlreadyExistsException, MBeanException,
            InvalidTargetObjectTypeException, InstallationServiceErrorException {

        this.embeddedJmxSrvCon.registerInstallationService(new InstallationService(new InstallationServiceResults()));
        this.unloadInstaller("componentName");

    }

    @Override
    public boolean unloadInstaller(final String componentName) throws InstallationServiceErrorException {
        try {
            return this.jmxClient.getInstallationServiceClient().unloadInstaller(componentName);
        } catch (final InstallationServiceDoesNotExistException e) {
            throw new InstallationServiceErrorException(e);
        } catch (final ConnectionErrorException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

}
