/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;

/**
 * Unit test of {@link PetalsJmxApiFactory} methods
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsJmxApiFactoryTest {

    /**
     * Port of the server protected with credentials
     */
    private final static Integer JMX_PORT_PROTECTED_SERVER = Integer.valueOf(7070);

    /**
     * Port of the server not protected by credentials
     */
    private final static Integer JMX_PORT_NOT_PROTECTED_SERVER = Integer
            .valueOf(JMX_PORT_PROTECTED_SERVER.intValue() + 1);

    private final static String JMX_HOST = "localhost";

    /**
     * A user having a non-empty password
     */
    private final static String JMX_USER = "petals";

    /**
     * A user having an empty password
     */
    private final static String JMX_USER_EMPTY_PASSWORD = "user-empty-password";

    private final static String JMX_PASSWORD = "petals";

    @Rule
    public final EmbeddedJmxServerConnector embeddedJmxSrvConProtected = new EmbeddedJmxServerConnector(
            JMX_PORT_PROTECTED_SERVER.intValue(), new String[][] { { JMX_USER, JMX_PASSWORD },
                    { JMX_USER_EMPTY_PASSWORD, null } });

    @Rule
    public final EmbeddedJmxServerConnector embeddedJmxSrvConNotProtected = new EmbeddedJmxServerConnector(
            JMX_PORT_NOT_PROTECTED_SERVER.intValue(), null);

    @Rule
    public final EmbeddedJmxServerConnector embeddedJmxSrvConDefault = new EmbeddedJmxServerConnector();

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>all connection parameters are correct</li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * </ul>
     * </p>
     */
    @Test
    public void testCreateJMXClient_AllIsOk() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, IOException, NotRemoteJMXClientException {

        final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST,
                JMX_PORT_PROTECTED_SERVER.intValue(), JMX_USER, JMX_PASSWORD);
        try {
            assertEquals("Unexpected hostname", JMX_HOST, jmxClient.getHost().getHostName());
            assertEquals("Unexpected port", JMX_PORT_PROTECTED_SERVER.intValue(), jmxClient.getPort());
            assertEquals("Unexpected user", JMX_USER, jmxClient.getUsername());
        } finally {
            jmxClient.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>the remote host name is set to <code>null</code></li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * <li>the host used is <code>localhost</code></li>
     * </ul>
     * </p>
     */
    @Test
    public void testCreateJMXClient_NullHost() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, IOException, NotRemoteJMXClientException {

        final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(null,
                JMX_PORT_PROTECTED_SERVER.intValue(), JMX_USER, JMX_PASSWORD);
        try {
            assertEquals("Unexpected hostname", JMX_HOST, jmxClient.getHost().getHostName());
            assertEquals("Unexpected port", JMX_PORT_PROTECTED_SERVER.intValue(), jmxClient.getPort());
            assertEquals("Unexpected user", JMX_USER, jmxClient.getUsername());
        } finally {
            jmxClient.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>the remote host port is set to <code>null</code></li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * <li>the host port used is the default one</li>
     * </ul>
     * </p>
     */
    @Test
    public void testCreateJMXClient_NullPort() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, IOException, NotRemoteJMXClientException {

        final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST, null, JMX_USER,
                JMX_PASSWORD);
        try {
            assertEquals("Unexpected hostname", JMX_HOST, jmxClient.getHost().getHostName());
            assertEquals("Unexpected port", JMXClient.DEFAULT_PORT, jmxClient.getPort());
            assertEquals("Unexpected user", JMX_USER, jmxClient.getUsername());
        } finally {
            jmxClient.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>the username is set to <code>null</code></li>
     * <li>the server is protected by credentials</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid password
     * </p>
     */
    @Test(expected = ConnectionErrorException.class)
    public void testCreateJMXClient_NullUserWithServerProtected() throws ConnectionErrorException,
            DuplicatedServiceException, MissingServiceException, IOException, NotRemoteJMXClientException {

        PetalsJmxApiFactory.getInstance().createJMXClient(JMX_USER, JMX_PORT_PROTECTED_SERVER, null, JMX_PASSWORD);
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>the username is set to <code>null</code></li>
     * <li>the server is <b>not</b> protected by credentials</li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * </ul>
     * </p>
     */
    @Test
    public void testCreateJMXClient_NullUserWithServerUnprotected() throws ConnectionErrorException,
            DuplicatedServiceException, MissingServiceException, IOException, NotRemoteJMXClientException {

        final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST,
                JMX_PORT_NOT_PROTECTED_SERVER, null, null);
        try {
            assertEquals("Unexpected hostname", JMX_HOST, jmxClient.getHost().getHostName());
            assertEquals("Unexpected port", JMX_PORT_NOT_PROTECTED_SERVER.intValue(), jmxClient.getPort());
        } finally {
            jmxClient.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>a valid username</li>
     * <li>a password set to null</li>
     * <li>the valid username requires a password</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid password
     * </p>
     */
    @Test(expected = ConnectionErrorException.class)
    public void testCreateJMXClient_NullPasswordRequired() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, IOException, NotRemoteJMXClientException {

        PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST, JMX_PORT_PROTECTED_SERVER.intValue(), JMX_USER,
                null);
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>a valid username</li>
     * <li>a password set to null</li>
     * <li>the valid username has an empty password</li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * </ul>
     * </p>
     */
    @Test
    public void testCreateJMXClient_NullPasswordNotRequired() throws ConnectionErrorException,
            DuplicatedServiceException, MissingServiceException, IOException, NotRemoteJMXClientException {

        final JMXClient jmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST,
                JMX_PORT_PROTECTED_SERVER.intValue(), JMX_USER_EMPTY_PASSWORD, null);
        try {
            assertEquals("Unexpected hostname", JMX_HOST, jmxClient.getHost().getHostName());
            assertEquals("Unexpected port", JMX_PORT_PROTECTED_SERVER.intValue(), jmxClient.getPort());
            assertEquals("Unexpected user", JMX_USER_EMPTY_PASSWORD, jmxClient.getUsername());
        } finally {
            jmxClient.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>a valid username</li>
     * <li>an invalid password</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid password
     * </p>
     */
    @Test(expected = ConnectionErrorException.class)
    public void testCreateJMXClient_InvalidPassword() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, IOException, NotRemoteJMXClientException {

        PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST, JMX_PORT_PROTECTED_SERVER.intValue(), JMX_USER,
                "invalid password");
    }

    /**
     * <p>
     * Check the creation of a {@link JMXClient} against:
     * <ul>
     * <li>an invalid username</li>
     * <li>a valid password</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid password
     * </p>
     */
    @Test(expected = ConnectionErrorException.class)
    public void testCreateJMXClient_InvalidUser() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, IOException, NotRemoteJMXClientException {

        PetalsJmxApiFactory.getInstance().createJMXClient(JMX_HOST, JMX_PORT_PROTECTED_SERVER.intValue(),
                "invalid_user", JMX_PASSWORD);
    }

}
